package br.edu.fei.dcc.raciocinioespacial.editor.application;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;

import br.edu.fei.dcc.raciocinioespacial.editor.commands.OpenScenaryAction;
import br.edu.fei.dcc.raciocinioespacial.editor.commands.OpenSimAction;
import br.edu.fei.dcc.raciocinioespacial.editor.commands.SaveSimAction;
import br.edu.fei.dcc.raciocinioespacial.editor.commands.RunSimAction;

public class ApplicationActionBarAdvisor extends ActionBarAdvisor {

	private IWorkbenchAction exitAction;
	private IAction openScenaryAction;
	private IAction saveSimAction;
	private IAction openSimAction;
	private IAction runSimAction;
	
    public ApplicationActionBarAdvisor(IActionBarConfigurer configurer) {
        super(configurer);
    }

    protected void makeActions(IWorkbenchWindow window) {
		this.openScenaryAction = new OpenScenaryAction(window);
		this.saveSimAction = new SaveSimAction(window);
		this.openSimAction = new OpenSimAction(window);
		this.exitAction = ActionFactory.QUIT.create(window);
		this.exitAction.setText("Exit");
		this.exitAction.setImageDescriptor(ImageDescriptor.createFromFile(Application.class, "icons/exit.png"));
		this.runSimAction = new RunSimAction(window);
		this.register(openScenaryAction);
		this.register(saveSimAction);
		this.register(openSimAction);
		this.register(exitAction);
		this.register(runSimAction);
    }

    protected void fillMenuBar(IMenuManager menuBar) {
		MenuManager fileMenu = 
			new MenuManager("&File",IWorkbenchActionConstants.M_FILE);
		menuBar.add(fileMenu);
		fileMenu.add(this.openScenaryAction);
		fileMenu.add(this.openSimAction);
		fileMenu.add(this.saveSimAction);
		fileMenu.add(this.exitAction);
		MenuManager simulacaoMenu =
			new MenuManager("&Simulation", IWorkbenchActionConstants.M_EDIT);
		simulacaoMenu.add(new Separator());
		simulacaoMenu.add(this.runSimAction);
		menuBar.add(simulacaoMenu);
    }

	@Override
	protected void fillCoolBar(ICoolBarManager coolBar) {
        IToolBarManager toolbar = new ToolBarManager(SWT.FLAT | SWT.CENTER);
        coolBar.add(new ToolBarContributionItem(toolbar, "main"));   
        toolbar.add(this.openScenaryAction);
        toolbar.add(this.openSimAction);
        toolbar.add(this.saveSimAction);
        toolbar.add(new Separator());
        toolbar.add(this.runSimAction);
	}
}