package br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements;
import org.eclipse.swt.graphics.Image;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.NumericProperty;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.NumericPropertyTypeEnum;


/**
 * The shape for the agent.
 */
public class Agent2DShape extends Vehicle2DShape {

	private static final String Property_Depth = "shapes.depth";
	//private static final String Property_Angle = "shapes.angle";
	private static final long serialVersionUID = 1;
	private int iDepth;
	private float fAngle;
	
	@Override
	/**
	 * Creates all editable properties for design time
	 */
	protected void criaPropriedades() {
		this.getDescriptors().add(new NumericProperty(Property_Depth, "Depth", NumericPropertyTypeEnum.Integer));
		// We omit this functionality because the angle have to be 120
		//this.getDescriptors().add(new NumericProperty(Property_Angle, "Angle", NumericPropertyTypeEnum.Float));
		super.criaPropriedades();
	}
	
	/** Returns the angle of observer's field vision (120) */
	public float getAngulo() {
		return fAngle;
	}
	
	@Override
	/** Returns the image icon used to represent the agent/observer into editor */
	public Image getIcon() {
		return super.createImage("icons/observador.png");
	}
	
	/** Returns the maximum observer's vision depth */
	public int getProfundidade() {
		return iDepth;
	}
	
	@Override
	/** Returns the property value specified by user */
	public Object getPropertyValue(Object propertyId) {
		if (Property_Depth.equals(propertyId)) {
			return Integer.toString(this.getProfundidade());
		}
		/*if (Property_Angle.equals(propertyId)) {
			return Float.toString(this.getAngulo());
		}*/
		return super.getPropertyValue(propertyId);
	}

	@Override
	/** Initiate the object with default configuration */
	protected void inicia() {
		super.inicia();
		this.iDepth = 600;
		this.fAngle = 120;
		this.name = "Agent";
	}

	/** Set the angle */
	public void setAngulo(float angulo) {
		this.fAngle = angulo;
	}

	/** Set the depth chosen by user */
	public void setProfundidade(int profundidade) {
		this.iDepth = profundidade;
	}

	@Override
	/** Set property values */
	public void setPropertyValue(Object propertyId, Object value) {
		if (Property_Depth.equals(propertyId)) {
			int profundidade = Integer.parseInt((String)value);
			this.setProfundidade(profundidade);
		}
		/*if (Property_Angle.equals(propertyId)) {
			float angulo = Float.parseFloat((String)value);
			this.setAngulo(angulo);
		}*/
		super.setPropertyValue(propertyId, value);
	}

	@Override
	/** Returns the name typed by user for the object */
	public String toString() {
		return this.getName();
	}
}
