package br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Image;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.NumericProperty;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.NumericPropertyTypeEnum;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.TextProperty;


/** The shape for the point used to trace object routes */
public class Ponto2DShape extends Shape {

	private static final String Property_NomePai = "shapes.pai";
	private static final String Property_Instant = "shapes.instante";
	private static final long serialVersionUID = 1;
	private static int iCount = 1;
	private Mobile2DShape pai;
	
	@Override
	/**
	 * Creates all editable properties for design time
	 */
	protected void criaPropriedades() {
		this.getDescriptors().add(new TextProperty(Property_NomePai, "Parent Entity"));
		this.getDescriptors().add(new NumericProperty(Property_Instant, "Instant", NumericPropertyTypeEnum.Integer));
		super.criaPropriedades();
	}
	
	/** Returns the image icon used to represent a point */
	public Image getIcon() {
		return super.createImage("icons/ponto.png");
	}

	@Override
	/** Returns the property value specified by user */
	public Object getPropertyValue(Object propertyId) {
		if (Property_NomePai.equals(propertyId)) {
			return this.getNomePai();
		}
		if (Property_Instant.equals(propertyId)) {
			return Integer.toString(this.getInstant());
		}
		return super.getPropertyValue(propertyId);
	}

	@Override
	/** Initiate the object with default configuration */
	protected void inicia() {
		super.inicia();
		super.name = "pt"+iCount;
		iCount++;
		super.setSize(new Dimension(3,3));
	}

	@Override
	/** Set property values */
	public void setPropertyValue(Object propertyId, Object value) {
		if (Property_NomePai.equals(propertyId)) {
			this.setNomePai((String)value);
		}
		super.setPropertyValue(propertyId, value);
	}

	/** Returns the name typed by user for the object */
	public String toString() {
		return this.getName();
	}

	/** Returns the name of the point's parent object */
	public String getNomePai() {
		return this.pai.getName();
	}

	/** Set the name of the point's parent object */
	public void setNomePai(String nomePai) {
		this.pai.setNome(nomePai);
	}

	@Override
	/** Set the point's size */
	public void setSize(Dimension newSize) {
		return; // It's not possible change the element size!
	}

	@Override
	/** Set the name for the point route */
	public void setNome(String name) {
		return; // It's not possible change the name!
	}

	/** Returns the instant of the point into the all route context. It works like an index of route points */
	public int getInstant() {
		return this.pai.getPoints().indexOf(this);
	}

	/** Set the parent object of the route point */
	public void setParent(Mobile2DShape parent) {
		this.pai = parent;
	}

	/** Set the parent object of the route point */
	public Mobile2DShape getPai() {
		return pai;
	}
}
