package br.edu.fei.dcc.raciocinioespacial.editor.shapes.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

import org.eclipse.ui.views.properties.IPropertyDescriptor;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Shape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.ListProperty;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.NumericProperty;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.NumericPropertyTypeEnum;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.PropertyDescriptorComparer;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.TextProperty;

/**
 * Um container para mltiplas formas/Shapes.
 * Este  a "raz" do modelo de estrutura de dados.
 */
public class ShapesDiagram extends ModelElement {

	/** Property ID used when a child is added into the diagram. */
	public static final String CHILD_ADDED_PROP = "ShapesDiagram.ChildAdded";
	/** Property ID used when a child is removed from the diagram. */
	public static final String CHILD_REMOVED_PROP = "ShapesDiagram.ChildRemoved";
	private static final long serialVersionUID = 1;
	public static final String Propriedade_Nome = "diagram.nome";
	public static final String Propriedade_Perfil = "diagram.perfil";
	public static final String Propriedade_Roads = "diagram.roads";
	public static final String Propriedade_Detail = "diagram.detail";
	public static final String Propriedade_Intervalo = "diagram.intervalo";
	private Collection<Shape> shapes = new Vector<Shape>();
	private String nome;
	private boolean showPerfilProfundidade;
	private boolean showDlgDetalhes;
	private boolean paintRoads;
	private float intervalo;
	protected transient List<IPropertyDescriptor> descriptors;

	public ShapesDiagram()
	{
		this.inicia();
		this.criaPropriedades();
	}
	
	public ShapesDiagram(String name) {
		this();
		this.setNome(name);
	}

	/** 
	 * Adiciona uma forma/Shape no diagrama.
	 * @param s uma instncia no-nula de Shape.
	 * @return Verdadeiro, se a forma/Shape foi adicionada com sucesso, Falso caso contrrio.
	 */
	public boolean addChild(Shape s) {
		if (s != null && shapes.add(s)) {
			firePropertyChange(CHILD_ADDED_PROP, null, s);
			return true;
		}
		return false;
	}

	protected void criaPropriedades() {
		this.getDescriptors().add(new TextProperty(Propriedade_Nome, "Name"));
		this.getDescriptors().add(new ListProperty(Propriedade_Perfil, "Show profile", new String[] {"No", "Yes"}));
		this.getDescriptors().add(new ListProperty(Propriedade_Detail, "Show details", new String[] {"No", "Yes"}));
		this.getDescriptors().add(new ListProperty(Propriedade_Roads, "Paint roads", new String[] {"No", "Yes"}));
		this.getDescriptors().add(new NumericProperty(Propriedade_Intervalo, "Interval between frames (ms)", NumericPropertyTypeEnum.Float));
	}

	public boolean existName(String name)
	{
		for(Shape s : this.getChildren())
		{
			if(s.getName().equals(name))
				return true;
		}
		return false;
	}
	
	/** Retorna a lista de formas/Shape contidas no diagrama. A lista retornada no pode ser modificada. */
	public List<Shape> getChildren() {
		return new Vector<Shape>(shapes);
	}
	
	public List<Shape> getChildren(Class<?> t)
	{
		List<Shape> retorno = new Vector<Shape>();
		
		for(Shape s : this.getChildren())
		{
			if(s.getClass().equals(t))
				retorno.add(s);
		}
		return retorno;
	}
	
	public List<String> getChildrensNames(Class<?> t)
	{
		List<String> retorno = new Vector<String>();
		List<Shape> shapes = this.getChildren(t);
		
		for(Shape s : shapes)
		{
			retorno.add(s.getName());
		}
		return retorno;
	}
	
	public List<IPropertyDescriptor> getDescriptors() {
		if(this.descriptors==null)
			this.setDescriptors(new ArrayList<IPropertyDescriptor>());
		return this.descriptors;
	}
	
	public float getIntervalo() {
		return intervalo;
	}

	public String getNome() {
		return nome;
	}
	
	public boolean getIsShowDlgDetalhes() {
		return showDlgDetalhes;
	}
	
	public boolean getIsPaintRoads() {
		return paintRoads;
	}
	
	public boolean getPerfilProfundidade() {
		return showPerfilProfundidade;
	}
	
	public IPropertyDescriptor[] getPropertyDescriptors() {
		IPropertyDescriptor[] retorno = new IPropertyDescriptor[this.getDescriptors().size()];
		Collections.sort(this.getDescriptors(), new PropertyDescriptorComparer());
		this.getDescriptors().toArray(retorno);
		return retorno;
	}

	@Override
	public Object getPropertyValue(Object propertyId) {
		if (Propriedade_Nome.equals(propertyId)) {
			return this.getNome();
		}
		if (Propriedade_Perfil.equals(propertyId)) {
			return (this.getPerfilProfundidade()?1:0);
		}
		if (Propriedade_Detail.equals(propertyId)) {
			return (this.getIsShowDlgDetalhes()?1:0);
		}
		if (Propriedade_Roads.equals(propertyId)) {
			return (this.getIsPaintRoads()?1:0);
		}
		if (Propriedade_Intervalo.equals(propertyId)) {
			return Float.toString(this.getIntervalo());
		}
		return super.getPropertyValue(propertyId);
	}

	private void inicia() {
		this.showDlgDetalhes = false;
		this.showPerfilProfundidade = true;
		this.paintRoads = false;
		this.nome = "Nova simulao";
		this.intervalo = 100.0f;
	}

	/**
	 * Remove a forma/Shape do diagrama.
	 * @param s uma instncia no-nula de Shape;
	 * @return Verdadeiro, se o Shape foi removido com sucesso, False caso contrrio.
	 */
	public boolean removeChild(Shape s) {
		if (s != null && shapes.remove(s)) {
			firePropertyChange(CHILD_REMOVED_PROP, null, s);
			return true;
		}
		return false;
	}

	public void setDescriptors(List<IPropertyDescriptor> descriptors) {
		this.descriptors = descriptors;
	}

	public void setIntervalo(float intervalo) {
		this.intervalo = intervalo;
		firePropertyChange(Propriedade_Intervalo, null, intervalo);
	}

	public void setNome(String nome) {
		this.nome = nome;
		firePropertyChange(Propriedade_Nome, null, nome);
	}

	public void setIsShowDlgDetalhes(boolean isDetalhes) {
		this.showDlgDetalhes = isDetalhes;
		firePropertyChange(Propriedade_Detail, null, isDetalhes);
	}
	
	public void setIsPaintRoads(boolean isPaintRoads) {
		this.paintRoads = isPaintRoads;
		firePropertyChange(Propriedade_Roads, null, isPaintRoads);
	}

	public void setPerfilProfundidade(boolean perfilProfundidade) {
		this.showPerfilProfundidade = perfilProfundidade;
		firePropertyChange(Propriedade_Perfil, null, perfilProfundidade);
	}

	@Override
	public void setPropertyValue(Object propertyId, Object value) {
		if (Propriedade_Nome.equals(propertyId)) {
			this.setNome((String)value);
		}
		if (Propriedade_Perfil.equals(propertyId)) {
			this.setPerfilProfundidade(((Integer)value).equals(1)?true:false);
		}
		if (Propriedade_Detail.equals(propertyId)) {
			this.setIsShowDlgDetalhes(((Integer)value).equals(1)?true:false);
		}
		if (Propriedade_Roads.equals(propertyId)) {
			this.setIsPaintRoads(((Integer)value).equals(1)?true:false);
		}
		if (Propriedade_Intervalo.equals(propertyId)) {
			this.setIntervalo(Float.parseFloat((String)value));
		}
		super.setPropertyValue(propertyId, value);
	}
	
}