package br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.command;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateRequest;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Agent2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Pedestrian2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Ponto2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Shape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Street2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Vehicle2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ShapesDiagram;

/**
 * Comando para adicionar uma forma/shape no ShapeDiagram.
 * Este comando pode ser desfeito e refeito.
 */
public class ShapeCreateCommand extends Command {
	/** A nova forma/shape. */ 
	private Shape newShape;

	/** ShapeDiagram onde os objetos sero adicionados. */
	private final ShapesDiagram parent;
	/** A requisio para criar uma nova forma/shape. */
	private final CreateRequest request;
	/** Verdadeiro, se o novo shape foi adicionado no diagrama. */
	private boolean shapeAdded;

	/**
	 * Cria o comando que adicionar a nova forma/shape no diagrama.
	 * @param parent 	uma instncia de ShapesDiagram que armazenar a forma.
	 * @param req     	quem est requisitando a criao da forma.
	 * @throws IllegalArgumentException se algum parmetro for nulo, ou o requisitante
	 * 						  no prover uma instncia de Shape.
	 */
	public ShapeCreateCommand(ShapesDiagram parent, CreateRequest req) {
		if (!this.validateCreateCommand(parent, req)) {
			throw new IllegalArgumentException(); // TODO Quase deu certo de validar se o carro esta na rua!
		}
		this.parent = parent;
		this.request = req;
		this.setLabel("Shape Creation");
	}

	private boolean validateCreateCommand(ShapesDiagram parent, CreateRequest req)
	{
		if (parent == null || req == null || !(req.getNewObject() instanceof Shape)) 
		{
			return false;
		}
		if(req.getNewObject() instanceof Agent2DShape)
		{
			if(parent.getChildren(Agent2DShape.class).size()>0)
			{
				return false;
			}
		}
		if(req.getNewObject() instanceof Vehicle2DShape)
		{
			// It verifies if the new object is inside a street
			boolean isInside = false;
			List<Shape> list = parent.getChildren(Street2DShape.class);
			for (Shape objShape : list) {
				Street2DShape streetAux =((Street2DShape)objShape);
				Rectangle RectStreet = new Rectangle(streetAux.getLocalization(), streetAux.getSize());
				if (RectStreet.contains(req.getLocation()))
				{
					isInside = true;
					break;
				}
			}
			return isInside;
		}
		if((req.getNewObject() instanceof Pedestrian2DShape) || (req.getNewObject() instanceof Ponto2DShape))
			return true;
		
		return false;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#canUndo()
	 */
	public boolean canUndo() {
		return this.shapeAdded;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		// Obtem a instncia da nova forma/shape a partir do requisitante.
		// Esta ao obriga a fbrica armazenada no requisitante a criar uma nova instncia.
		// Esta fbrica  abastecida pela paleta de entrada, veja
		// ShapesEditorPaletteFactory#createComponentsGroup()
		this.newShape = (Shape) request.getNewObject();
		// Obtem a localizao e o tamanho desejado para o objeto.
		this.newShape.setParent(parent);
		this.newShape.setSize(request.getSize());
		this.newShape.setLocalization(request.getLocation());
		this.newShape.setNome(this.getNomePadraoPorTipo(this.newShape));
		redo();
	}
	
	private String getNomePadraoPorTipo(Shape x)
	{
		List<String> nomes = new Vector<String>();
		String prefixoPadrao = new String();
		if(x instanceof Agent2DShape)
			nomes = this.parent.getChildrensNames(Agent2DShape.class);
		else if(x instanceof Vehicle2DShape)
			nomes = this.parent.getChildrensNames(Vehicle2DShape.class);
		else if(x instanceof Pedestrian2DShape)
			nomes = this.parent.getChildrensNames(Pedestrian2DShape.class);
		
		prefixoPadrao = x.getName();
		
		if(nomes.size()>0)
		{
			for(int i=0; i<nomes.size()+1; i++)
			{
				if(!nomes.contains(prefixoPadrao+(i+1)))
				{
					return prefixoPadrao+(i+1);
				}
			}
		}
		return prefixoPadrao + 1;	
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		this.shapeAdded = parent.addChild(newShape);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		this.parent.removeChild(newShape);
	}
}