package br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.command;

import org.eclipse.gef.commands.Command;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Mobile2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Ponto2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Shape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ShapesDiagram;

/**
 * Comando para remover a forma/shape de seu container.
 * Este comando pode ser desfeito e refeito.
 */
public class ShapeDeleteCommand extends Command {
	/** Forma/Shape a ser removida. */
	private final Shape child;

	/** ShapeDiagram onde os objetos sero removidos. */
	private final ShapesDiagram parent;
	/** Verdadeiro, se a forma/shape foi removida de seu container. */
	private boolean wasRemoved;

	/**
	 * Cria o comando que remover a forma/shape de seu container.
	 * @param parent 		uma instncia de ShapesDiagram onde a forma/shape est contida
	 * @param child    		a forma/shape a ser removida
	 * @throws IllegalArgumentException se algum parmetro for nulo.
	 */
	public ShapeDeleteCommand(ShapesDiagram parent, Shape child) {
		if (parent == null || child == null) {
			throw new IllegalArgumentException();
		}
		setLabel("Excluso de formas");
		this.parent = parent;
		this.child = child;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#canUndo()
	 */
	public boolean canUndo() {
		return wasRemoved;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() { 
		redo();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		// Remove a forma/shape.
		if(child instanceof Ponto2DShape)
		{
			Ponto2DShape pt = (Ponto2DShape)child;
			wasRemoved = pt.getPai().getPoints().remove(pt) && parent.removeChild(child);
		}else if(child instanceof Mobile2DShape){
			Mobile2DShape obj = (Mobile2DShape)child;
			wasRemoved = this.parent.removeChild(obj);
			for(Ponto2DShape pt : obj.getPoints())
			{
				wasRemoved = (wasRemoved && this.parent.removeChild(pt));
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		// Adiciona a forma/shape.
		parent.addChild(child);
		if(child instanceof Ponto2DShape)
		{
			Ponto2DShape pt = (Ponto2DShape)child;
			pt.getPai().getPoints().add(pt);
		}
	}
}