package br.edu.fei.dcc.raciocinioespacial.editor.shapes.palette;

import org.eclipse.gef.palette.CombinedTemplateCreationEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteContainer;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.PaletteSeparator;
import org.eclipse.gef.palette.PanningSelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gef.requests.SimpleFactory;
import org.eclipse.gef.ui.palette.FlyoutPaletteComposite.FlyoutPreferences;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;

import br.edu.fei.dcc.raciocinioespacial.editor.application.Activator;
import br.edu.fei.dcc.raciocinioespacial.editor.application.Application;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Agent2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Pedestrian2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Vehicle2DShape;

/**
 * Classe utilitria para criar a paleta GEF.
 * @see #createPalette() 
 */
public final class ShapesEditorPaletteFactory {

	/** Identificador de preferncias usada para persistir o local da paleta. */
	private static final String PALETTE_DOCK_LOCATION = "ShapesEditorPaletteFactory.Location";
	/** Identificador de preferncias usada para persistir o tamaho da paleta. */
	private static final String PALETTE_SIZE = "ShapesEditorPaletteFactory.Size";
	/** Identificador de preferncias usada para persistir o estado da paleta. */
	private static final String PALETTE_STATE = "ShapesEditorPaletteFactory.State";

	/** Create drawer tools for the palette */
	private static PaletteContainer createShapesDrawer() {
		PaletteDrawer componentsDrawer = new PaletteDrawer("Entities");

		// Creating the pedestrian for the pallete
		CombinedTemplateCreationEntry component = new CombinedTemplateCreationEntry(
				"Pedestrian", "Make a pedestrian",
				Pedestrian2DShape.class,
				new SimpleFactory(Pedestrian2DShape.class), 
				ImageDescriptor.createFromFile(Application.class, "icons/pedestrian.png"),
				ImageDescriptor.createFromFile(Application.class, "icons/pedestrian.png"));
		componentsDrawer.add(component);
		
		// Creating the vehicle for the pallete
		component = new CombinedTemplateCreationEntry(
				"Vehicle", "Make a vehicle",
				Vehicle2DShape.class,
				new SimpleFactory(Vehicle2DShape.class), 
				ImageDescriptor.createFromFile(Application.class, "icons/vehicle.png"),
				ImageDescriptor.createFromFile(Application.class, "icons/vehicle.png"));
		componentsDrawer.add(component);

		// Creating the agent for the pallete
		component = new CombinedTemplateCreationEntry(
				"Agent", "Make the agent", Agent2DShape.class,
				new SimpleFactory(Agent2DShape.class), 
				ImageDescriptor.createFromFile(Application.class, "icons/observador.png"),
				ImageDescriptor.createFromFile(Application.class, "icons/observador.png"));
		componentsDrawer.add(component);

		return componentsDrawer;
	}

	/**
	 * Cria a raz da paleta e adiciona todos os elementos da paleta.
	 * Utiliza esta fbrica para criar uma nova paleta para o editor grfico.
	 * @return uma nova PaletteRoot
	 */
	public static PaletteRoot createPalette() {
		PaletteRoot palette = new PaletteRoot();
		palette.add(createToolsGroup(palette));
		palette.add(createShapesDrawer());
		return palette;
	}

	/**
	 * Retorna uma instncia de FlyoutPreferences usada para salvar/carregar as preferncias da paleta "Flayout".
	 */
	public static FlyoutPreferences createPalettePreferences() {
		return new FlyoutPreferences() {
			private IPreferenceStore getPreferenceStore() {
				return Activator.getDefault().getPreferenceStore();
			}
			public int getDockLocation() {
				return getPreferenceStore().getInt(PALETTE_DOCK_LOCATION);
			}
			public int getPaletteState() {
				return getPreferenceStore().getInt(PALETTE_STATE);
			}
			public int getPaletteWidth() {
				return getPreferenceStore().getInt(PALETTE_SIZE);
			}
			public void setDockLocation(int location) {
				getPreferenceStore().setValue(PALETTE_DOCK_LOCATION, location);
			}
			public void setPaletteState(int state) {
				getPreferenceStore().setValue(PALETTE_STATE, state);
			}
			public void setPaletteWidth(int width) {
				getPreferenceStore().setValue(PALETTE_SIZE, width);
			}
		};
	}

	/** Create the "Tool" group. */
	private static PaletteContainer createToolsGroup(PaletteRoot palette) {
		PaletteGroup toolGroup = new PaletteGroup("Tools");

		// Add a 'Select' tool for the group
		ToolEntry tool = new PanningSelectionToolEntry();
		toolGroup.add(tool);
		palette.setDefaultEntry(tool);

		// Add a 'marquee' tool for the group
		toolGroup.add(new MarqueeToolEntry());

		// Add a separator for the group
		toolGroup.add(new PaletteSeparator());

		return toolGroup;
	}

	/** Classe utilitria. */
	private ShapesEditorPaletteFactory() {
	}
}