package br.edu.fei.dcc.raciocinioespacial.editor.shapes.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.AbstractTreeEditPart;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Ponto2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Shape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ModelElement;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ShapesDiagram;


/**
 * TreeEditPart para uma instncia de ShapesDiagram.
 * Isto  usado na viso Outline do ShapesEditor.
 * <p>Este edit part deve implementar a interface PropertyChangeListener, 
 * sendo assim, ser notificada quando houverem mudanas nas propriedades no model element correspondente.
 * </p>
 */
class DiagramTreeEditPart extends AbstractTreeEditPart implements PropertyChangeListener {

	/** 
	 * Cria uma nova instncia deste edit part usando os elementos do modelo.
	 * @param model a non-null ShapesDiagram instance
	 */
	DiagramTreeEditPart(ShapesDiagram model) {
		super(model);
	}

	/**
	 * Aps a ativao,  anexado no model element como um listener de modificaes das propriedades.
	 */
	public void activate() {
		if (!isActive()) {
			super.activate();
			((ModelElement) getModel()).addPropertyChangeListener(this);
		}
	}

	/* (non-Javadoc)
	 * @see br.edu.fei.dcc.raciocinioespacial.editor.shapes.parts.ShapeTreeEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// If this editpart is the root content of the viewer, then disallow removal
		if (getParent() instanceof RootEditPart) {
			installEditPolicy(EditPolicy.COMPONENT_ROLE, new RootComponentEditPolicy());
		}
	}

	/**
	 * Aps a desativao,  desanexado do model element como um listener de modificaes das propriedades.
	 */
	public void deactivate() {
		if (isActive()) {
			super.deactivate();
			((ModelElement) getModel()).removePropertyChangeListener(this);
		}
	}

	private ShapesDiagram getCastedModel() {
		return (ShapesDiagram) getModel();
	}

	/**
	 * Mtodo que retorna a EditPart correspondente com a forma child.
	 * @param child 		uma instncia de elemento do modelo.
	 * @return 				a EditPart correspondente ou nulo.
	 */
	private EditPart getEditPartForChild(Object child) {
		return (EditPart) getViewer().getEditPartRegistry().get(child);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		return getCastedModel().getChildren(); // a list of shapes
	}

	private int getNewIndex(Object obj)
	{
		if(obj instanceof Ponto2DShape)
		{
			//Procura pelo n pai
			int indexPai = 0;
			for(indexPai=0; indexPai<this.children.size(); indexPai++)
			{
				if(((Shape)((ShapeTreeEditPart)this.children.get(indexPai)).getModel()).getName().equals(((Ponto2DShape)obj).getNomePai()))
				{
					break;
				}
			}
			return indexPai + ((Ponto2DShape)obj).getInstant() + 1;
		}
		return -1;
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
		if (ShapesDiagram.CHILD_ADDED_PROP.equals(prop)) {
			// Adiciona os elementos nessa editpart
			// adiciona uma entrada que aparecer na rvore da viso Outline.
			this.addChild(createChild(evt.getNewValue()), this.getNewIndex(evt.getNewValue()));
		}
		else if (ShapesDiagram.CHILD_REMOVED_PROP.equals(prop)) {
			// Remove o elemento desta EditPart
			// remove uma entrada, retirando-a da rvore da viso Outline.
			this.removeChild(getEditPartForChild(evt.getNewValue()));
		}
		else {
			refreshVisuals();
		}
	}	
}