/**
 * 
 */
package br.edu.fei.dcc.raciocinioespacial.simulador.movimentacao;

import java.awt.Point;
import java.awt.Polygon;

/** 
 * @author au.edu.unsw.cse
 */
public class BezierSpline extends ControlCurve implements ICriadorMovimentacao {

	private static final int DEFAULT_STEPS = 180; 
	
	public BezierSpline(Polygon pts, int steps)
	{
		this.pts = pts;
		this.steps = steps;
	}
	
	public BezierSpline(Polygon pts)
	{
		this(pts, BezierSpline.DEFAULT_STEPS);
	}
	
	// the basis function for a Bezier spline
	static float b(int i, float t) {
		switch (i) {
		case 0:
			return (1-t)*(1-t)*(1-t);
		case 1:
			return 3*t*(1-t)*(1-t);
		case 2:
			return 3*t*t*(1-t);
		case 3:
			return t*t*t;
		}
		return 0; //we only get here if an invalid i is specified
	}

	//evaluate a point on the B spline
	Point p(int i, float t) {
		float px=0;
		float py=0;
		for (int j = 0; j<=3; j++){
			px += b(j,t)*pts.xpoints[i+j];
			py += b(j,t)*pts.ypoints[i+j];
		}
		return new Point(Math.round(px),Math.round(py));
	}

	/* (non-Javadoc)
	 * @see br.edu.fei.dcc.raciocionioespacial.simulador.movimentacao.ICriadorMovimentacao#criar(java.awt.Polygon)
	 */
	@Override
	public Polygon criar(Polygon pts) {
		Polygon pol = new Polygon ();
		Point q = p(0,0);
		pol.addPoint(q.x,q.y);
		for (int i = 0; i < pts.npoints-3; i+=3) {
			for (int j = 1; j <= this.steps; j++) {
				q = p(i,j/(float)this.steps);
				pol.addPoint(q.x,q.y);
			}
		}
		return pol;		
	}
	
	@Override
	public Polygon criar() {
		return this.criar(this.pts);
	}
}
