/**
 * 
 */
package br.edu.fei.dcc.raciocinioespacial.simulador.movimentacao;

import java.util.ArrayList;
import java.util.GregorianCalendar;

import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.IEntidadeMovel;
import br.edu.fei.dcc.raciocinioespacial.simulador.simulacao.Simulacao;
import br.edu.fei.dcc.raciocinioespacial.simulador.util.Utils;

public class GerenciadorMovimentacao implements Runnable {
	private Simulacao simulacao;
	private float fInterval;
	private boolean isToStop;
	private Thread thread;
	
	private final int TAXA_ASSERTS = 10;
	
	/** It starts the thread which refreshes the movements constantly */
	public void startMovements()
	{
		this.thread = new Thread(this);
		this.thread.setPriority(Thread.MIN_PRIORITY);
		this.thread.start();
	}
	
	public void stopMovements()
	{
		this.isToStop = true;
	}
	
	public GerenciadorMovimentacao(Simulacao simulacao, float intervalo, ArrayList<IEntidadeMovel> entidades) {
		super();
		this.simulacao = simulacao;
		this.fInterval = intervalo;
	}

	/**
	 * It run the new thread
	 */
	public void run() {
		this.simulacao.notifyObservers("Simulation running...");
		long timeToWait = 0;
		this.isToStop = false;
		
		int iAssertsCount = TAXA_ASSERTS;
		
		long timeStartRun = GregorianCalendar.getInstance().getTimeInMillis();
		
		while(simulacao.getAgent().hasMovement())
		{
			long timeStartProc = GregorianCalendar.getInstance().getTimeInMillis();
			timeToWait += this.fInterval;
			
			// At the beginning of each movement cycle we need to update the draw of the streets
			simulacao.updateStreets();
			simulacao.updateTime(timeStartProc-timeStartRun);
			
			simulacao.getAgent().move();
						
			for(IEntidadeMovel iem : simulacao.getObjetos())
				iem.move();
			
			if (isToStop)
				break;
			
			if (iAssertsCount >= TAXA_ASSERTS)
			{
				//simulacao.plotarEsqueletoPerfilProfundidade();
				simulacao.getProfile().calcCampoVisaoInstantaneo();
				iAssertsCount = 0;
				System.out.println(simulacao.getProfile().getCurrentProfile());
			}
			iAssertsCount++;
			
			timeToWait-=(GregorianCalendar.getInstance().getTimeInMillis()-timeStartProc);
			if(timeToWait>0)
			{
				Utils.aguardar(timeToWait);
				timeToWait = 0;
			}
		}
		
		// It only resets the simulation if it has reached the end,
		// not when the user has pressed the stop/pause button
		if (!isToStop)
		{
			simulacao.endSimulation();
			simulacao.getAgent().moveInitialPos();
			for(IEntidadeMovel iem : simulacao.getObjetos())
			{
				iem.moveInitialPos();
			}
		}
	}
}