package br.edu.fei.dcc.raciocinioespacial.simulador.perfilprofundidade;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Arc2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Agent2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Pedestrian2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.util.Utils;

public class CampoVisaoInstantaneo {
	private Agent2D observador;
	private List<ObstaculoCampoVisao> objetos;
	private int iTime = 0;
	
	public CampoVisaoInstantaneo(Agent2D observador, List<Pedestrian2D> objetos, int time) {
		super();
		this.observador = observador;
		this.objetos = this.getObjetosDentroCampoVisao(objetos);
		this.iTime = time;
		this.criar();
	}
	private List<ObstaculoCampoVisao> getObjetosDentroCampoVisao(List<Pedestrian2D> objetos) {
		Arc2D obs = this.observador.getCampoVisao2D_Old();
		List<ObstaculoCampoVisao> retorno = new ArrayList<ObstaculoCampoVisao>();
		
		for(Pedestrian2D o : objetos)
		{
			Rectangle rectO = o.getRect();
			if(obs.intersects(rectO))
			{
				ObstaculoCampoVisao obst = new ObstaculoCampoVisao(o, rectO);
				retorno.add(obst);
			}
		}
		return retorno;
	}
	private void criar() {
		float initialAngle = (float)this.observador.getCampoVisao2D_Old().getAngleStart();
		float finalAngle   = initialAngle + (float)this.observador.getCampoVisao2D_Old().getAngleExtent();
		
		Point ptObs = new Point((int)observador.getObjeto2D().getCenterX(),(int)observador.getObjeto2D().getCenterY());
		
		// For each object inside the observer's field of vision, we set an obstacle type
		for (ObstaculoCampoVisao obst : objetos) {
			Rectangle rectPed = obst.getRect();
			Point ptA = new Point((int)rectPed.getCenterX(),(int)rectPed.getCenterY());
			float dDistance = Utils.convertPixelsToMeters((float)(ptObs.distance(ptA)));
			float fAngleRef = (float)Math.toDegrees(Math.atan((ptA.getY()-ptObs.getY())/(ptA.getX()-ptObs.getX())));
			fAngleRef = 180 - fAngleRef;
			if (fAngleRef < 0)
				fAngleRef = 360 + fAngleRef;
			
			fAngleRef = fAngleRef - initialAngle;
			
			float fAngleOpen = (float)Math.toDegrees(Utils.convertPixelsToMeters(rectPed.getWidth())/dDistance);
			
			obst.setAngles(fAngleOpen, fAngleRef);
			obst.setProfundidade(dDistance);
			
//			System.out.println(String.format("PedX[x=%s,y=%s,a=%s,d=%s]", ""+ptA.x, ""+ptA.y, ""+fAngleOpen, ""+dDistance));
		}
		/*for(float teta=anguloIncial; teta<=anguloFinal; teta++)
		{
			parteCampo.add(new ParteCampoVisaoInstantaneo(this.instante, teta, observador, objetos));
		}*/
	}
	public int getTimeInstant() {
		return iTime;
	}
	
	public void sortByReferenceAngle()
	{
		// A simply bubble sort !
		int iSize = objetos.size();
		for (int i = 0; i < iSize-1; i++)
		{
			for (int j = 0; j < iSize-i-1; j++) {
				if (objetos.get(j).getReferenceAngle() > objetos.get(j+1).getReferenceAngle())
				{
					ObstaculoCampoVisao oTemp = objetos.get(j);
					objetos.set(j, objetos.get(j+1));
					objetos.set(j+1, oTemp);
			    }
			}
		}
	}
	
	public List<ObstaculoCampoVisao> getObstaculos() {
		return objetos;
	}
}
