package br.edu.fei.dcc.raciocinioespacial.simulador.perfilprofundidade;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.List;

import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Agent2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Pedestrian2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.util.Utils;

public class PerfilProfundidade {
	private static final String INICIO = "leftBorder";
	private static final String FIM = "rightBorder";
	private static final String EXTENSAO = "pl";
	private static final String CABECALHO = "description";
	private static final String BACKGROUND = "background";
	private static final String FOREGROUND = "foreground";
	
	private Agent2D observador;
	private List<Pedestrian2D> objetos;
	private CampoVisaoInstantaneo campoVisao;
	private String sCurrentProfile = "";
	private StringBuilder sbAllProfile = new StringBuilder();
	private Graphics graphProfile;
	private float fCurrentAngle = 0;
	private float fatorX = 0;
	private float fatorY = 0;
	private Rectangle rectDepthProfile;
	private Rectangle rectLimiteProfile;
	
	private static int iCountCampo = 0;
	
	public PerfilProfundidade(Agent2D observador, List<Pedestrian2D> objetos,
			Graphics g, Rectangle rectDepthProfile, Rectangle rectLimiteProfile) {
		super();
		this.observador = observador;
		this.rectDepthProfile = rectDepthProfile;
		this.rectLimiteProfile = rectLimiteProfile;
		this.graphProfile = g;
		this.objetos = objetos;
		
		this.fatorX = (float)(rectDepthProfile.width/(float)(observador.getAngulo()));
		this.fatorY = (float)(rectDepthProfile.height/(float)(observador.getProfundidade()));
	}
	public List<Pedestrian2D> getObjetos() {
		return objetos;
	}
	public Agent2D getObservador() {
		return observador;
	}
	
	public CampoVisaoInstantaneo getCampoVisaoInstantaneo()
	{
		if (campoVisao == null)
			calcCampoVisaoInstantaneo();
		
		return campoVisao;
	}
	
	private void makeDepthProfile()
	{
		sCurrentProfile = "";
		fCurrentAngle = 0;
		plotarEsqueletoPerfilProfundidade();
		// Para cada obtculo devidamente calculado no campoVisao
		// vou criando o perfil de profundidade contnuo
		
		// Ordeno por angulo de referencia
		campoVisao.sortByReferenceAngle();
		
		// Vou criando o perfil, sempre verificando se o objeto atual no est em relao de 
		// ocluso com o objeto anterior
		ObstaculoCampoVisao prevObstatulo = null;
		for (ObstaculoCampoVisao obstaculo : campoVisao.getObstaculos()) {
			// No caso de ter ocluso, verifico quem est mais perto do agente e
			// retiro a diferenca entre o angulo inicial de um com o final do outro
			// de forma que o angulo de abertura passa a desconsiderar a parte q o agente
			// nao estaria msm vendo de fato
			if (prevObstatulo != null)
				prevObstatulo.trataOclusion(obstaculo);
			
			buildPartProfile(obstaculo, prevObstatulo);
			
			// Se algum obstculo acabar ficando com 0 graus de angulo de abertura,
			// o msm deve ser ignorado pois est totalmente escondido atrs de um ou mais
			// obstculos
			if (obstaculo.getOpeningAngle() > 0)
				prevObstatulo = obstaculo;
		}
		
		endStringProfile(prevObstatulo);
		
		/*if((int)(instante/this.fInterval)<this.profile.getCampoInstantaneo().size())
		if((int)(5)<this.profile.getCampoInstantaneo().size())
		{
			CampoVisaoInstantaneo c = this.profile.getCampoInstantaneo().get((int)(this.fInterval));
			
			
			int x0 = limitGraphicProfile.x;
			int y0 = limitGraphicProfile.y;
			int qtd = limitGraphicProfile.width/c.getParteCampo().size();
			List<Pedestrian2D> obstaculos = new ArrayList<Pedestrian2D>();
			
			for(int i=1; i<=c.getParteCampo().size(); i++)
			{
				ParteCampoVisaoInstantaneo p = c.getParteCampo().get(i-1);
				
				for(int j=0; j<qtd; j++)
				{
					this.getContextProfile().fillOval((int)(x0+(qtd*(i-1)-j)), (int)(y0+(p.getProfundidade()*fator)), 1, 1);
					if(p.getInstersecaoComObservador()!=null)
					{
						if(!obstaculos.contains(p.getInstersecaoComObservador()))
						{
							obstaculos.add(p.getInstersecaoComObservador());
							this.getContextProfile().drawString(p.getInstersecaoComObservador().getNome(), (int)(x0+(qtd*(i-1))), (int)(y0+(p.getProfundidade()*fator)));
						}
						if(i<c.getParteCampo().size())
							if(c.getParteCampo().get(i).getInstersecaoComObservador()!=null)
								this.getContextProfile().drawLine((int)(x0+(qtd*(i-1))), (int)(y0+(p.getProfundidade()*fator)), (int)(x0+(qtd*(i))), (int)(y0+(c.getParteCampo().get(i).getProfundidade()*fator)));
							else
								this.getContextProfile().drawLine((int)(x0+(qtd*(i-1))), (int)(y0+(p.getProfundidade()*fator)), (int)(x0+(qtd*(i)))-qtd, (int)(y0+(c.getParteCampo().get(i).getProfundidade()*fator)));
						if((i-2)>=0)
							this.getContextProfile().drawLine((int)(x0+(qtd*(i-2))), (int)(y0+(c.getParteCampo().get(i-2).getProfundidade()*fator)), (int)(x0+(qtd*(i-1))), (int)(y0+(p.getProfundidade()*fator)));
						break;
					}
				}
			}*
		}*/
	}
	
	private void buildPartProfile(ObstaculoCampoVisao obst, ObstaculoCampoVisao prevObst)
	{
		float initialAngle = obst.getInitialAngle();
		float openningAngle = obst.getOpeningAngle();
		float depth = obst.getProfundidade();
		String sObstName = ""; //obst.getObstaculo().getNome();
		
		int iX1 = 0;
		int iY1 = rectDepthProfile.height;
		int iX2 = 0;
		int iY2 = 0;
		
		// Plotar rea de silencio
		if (initialAngle > fCurrentAngle)
		{
			iY2 = iY1;
			iX1 = (int)(fCurrentAngle * fatorX);
			iX2 = (int)(initialAngle * fatorX);
			graphProfile.drawLine(iX1, iY1, iX2, iY2);
			
			if (prevObst != null)
				sObstName = "("+prevObst.getObstaculo().getNome()+","+obst.getObstaculo().getNome()+")";
			else
				sObstName = "("+INICIO+","+obst.getObstaculo().getNome()+")";
			
			addStringProfile(sObstName, -1, initialAngle - fCurrentAngle);
		}
		
		iY1 = rectDepthProfile.height;
		iY2 = (int)(rectDepthProfile.height - (depth * fatorY));
		iX1 = iX2 = (int)(initialAngle * fatorX);
		graphProfile.drawLine(iX1, iY1, iX2, iY2);
		
		iY1 = iY2 = (int)(rectDepthProfile.height - (depth * fatorY));
		iX1 = (int)(initialAngle * fatorX);
		iX2 = (int)((initialAngle + openningAngle) * fatorX);
		graphProfile.drawLine(iX1, iY1, iX2, iY2);
		
		iY1 = rectDepthProfile.height;
		iY2 = (int)(rectDepthProfile.height - (depth * fatorY));
		iX1 = iX2 = (int)((initialAngle + openningAngle) * fatorX);
		graphProfile.drawLine(iX1, iY1, iX2, iY2);
		
		sObstName = obst.getObstaculo().getNome();		
		addStringProfile(sObstName, depth, openningAngle);
		
		fCurrentAngle = initialAngle + openningAngle;
	}
	
	private void addStringProfile(String nameObj, float depth, float angle)
	{
		if (sCurrentProfile == null || sCurrentProfile.length() == 0)
			sCurrentProfile = String.format("%s(t%d,[", CABECALHO, campoVisao.getTimeInstant());
		else
			sCurrentProfile += ",";
		
		if (depth == -1)
			sCurrentProfile += BACKGROUND + "(";
		else
			sCurrentProfile += FOREGROUND + "(";
		
		sCurrentProfile += nameObj+","+depth+","+angle+")";
	}
	
	private void endStringProfile(ObstaculoCampoVisao lastObj)
	{
		if (fCurrentAngle < observador.getAngulo())
		{
			float angle = observador.getAngulo() - fCurrentAngle; 
			String sNameObj = "("+lastObj.getObstaculo().getNome()+","+FIM+")";;
			addStringProfile(sNameObj, -1, angle);
		}
		
		sCurrentProfile += "])";
		sbAllProfile = sbAllProfile.append(sCurrentProfile + "\r\n");
	}
	
	private void plotarEsqueletoPerfilProfundidade()
	{
		if (this.graphProfile != null)
		{
			Graphics2D g2 = (Graphics2D)this.graphProfile;

			g2.setFont(new Font(Font.SERIF, Font.PLAIN, 12));
			g2.clearRect(0, 0, Short.MAX_VALUE, Short.MAX_VALUE);
			g2.setBackground(Color.WHITE);
			g2.draw(rectLimiteProfile);
			Utils.drawArrowLine(rectDepthProfile.x-5, rectDepthProfile.y, rectDepthProfile.x-5, rectDepthProfile.y+rectDepthProfile.height+5, 3, 3, g2);
			Utils.drawArrowLine(rectDepthProfile.x-10, rectDepthProfile.y+5, rectDepthProfile.x+rectDepthProfile.width-10, rectDepthProfile.y+5, 3, 3, g2);
		}
	}
	
	public void calcCampoVisaoInstantaneo()
	{
		campoVisao = new CampoVisaoInstantaneo(observador, objetos, iCountCampo);
		makeDepthProfile();
		iCountCampo++;
	}
	public String getCurrentProfile() {
		return sCurrentProfile;
	}
	public StringBuilder getAllProfile() {
		return sbAllProfile;
	}
}
