/**
 * 
 */
package br.edu.fei.dcc.raciocinioespacial.simulador.entidades;

import java.awt.Color;
import java.awt.Point;
import java.awt.geom.Arc2D;
import java.util.ArrayList;

import br.edu.fei.dcc.raciocinioespacial.simulador.util.SentidoVehicle;

/** 
 * @author Thiago
 */
public class Agent2D extends Vehicle2D {
	//private static final int FATOR_CORRECAO_DIRECAO = 12;
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * @uml.property  name="direcao"
	 */
	private float[] direcao;

	/**
	 * @uml.property  name="profundidade"
	 */
	private int profundidade;

	/**
	 * @uml.property  name="angulo"
	 */
	private float angulo;
	
	protected Arc2D arc2DOld;

	/**
	 */
	public Agent2D(int x, int y, int width, int height, String nome, int profundidade, float angulo){
		this(x, y ,width, height,nome, Color.YELLOW ,new ArrayList<Point>(), profundidade, angulo);
	}
	
	/**
	 */
	public Agent2D(int x, int y, int width, int height, String nome, Color cor, int profundidade, float angulo){
		this(x,y, width, height, nome,cor, new ArrayList<Point>(), profundidade, angulo);
	}
	
	public Agent2D()
	{
		this(0, 0, 0, 10, "obs", 100, 45);
	}
	
	/**
	 */
	public Agent2D(int x, int y, int width, int height, String nome, Color cor, ArrayList<Point> pontos, int profundidade, float angulo){
		super(x,y,width,height,nome,cor,pontos);
		this.setProfundidade(profundidade);
		this.setAngulo(angulo);
	}

	/**
	 * Getter of the property <tt>angulo</tt>
	 * @return  Returns the angulo.
	 * @uml.property  name="angulo"
	 */
	public float getAngulo() {
		return angulo;
	}

	/**
	 * Getter of the property <tt>direcao</tt>
	 * @return  Returns the direcao.
	 * @uml.property  name="direcao"
	 */
	public float[] getDirecao() {
		return direcao;
	}

	/**
	 * Getter of the property <tt>profundidade</tt>
	 * @return  Returns the profundidade.
	 * @uml.property  name="profundidade"
	 */
	public int getProfundidade() {
		return profundidade;
	}

	public Arc2D getCampoVisao2D()
	{
		float fPosXStart = 0f;
		float fPosYStart = 0f;
		float fAngStart = (180 - this.getAngulo()) / 2;
		
		// We verify what direction the agent is taking, and then we
		// can know what direction the "arc vision" have to be
		if (svSentido == SentidoVehicle.VERTICAL)
		{
			fPosXStart = xCurrent - this.getProfundidade()/2 + (iWidth/2);
			
			if (isPosIncreasing)
			{
				fPosYStart = yCurrent + iHeight - this.getProfundidade()/2;
				fAngStart += 180;
			}
			else
			{
				fPosYStart = yCurrent - this.getProfundidade()/2;
			}
		}
		else // SentidoVehicle.HORIZONTAL
		{
			fPosYStart = yCurrent - this.getProfundidade()/2 + (iHeight/2);
			
			if (isPosIncreasing)
			{
				fPosXStart = xCurrent + iWidth - this.getProfundidade()/2;
				fAngStart += 270;
			}
			else
			{
				fPosXStart = xCurrent - this.getProfundidade()/2;
				fAngStart += 90;
			}
		}
		
		arc2DOld = new Arc2D.Float(fPosXStart, fPosYStart,
				this.getProfundidade(), this.getProfundidade(), fAngStart,
				this.getAngulo(), Arc2D.PIE); 
		return arc2DOld;
	}
	
	public Arc2D getCampoVisao2D_Old()
	{
		return arc2DOld; 
	}
	
	/**
	 * Setter of the property <tt>angulo</tt>
	 * @param angulo  The angulo to set.
	 * @uml.property  name="angulo"
	 */
	public void setAngulo(float angulo) {
		this.angulo = angulo;
	}

	/**
	 * Setter of the property <tt>direcao</tt>
	 * @param direcao  The direcao to set.
	 * @uml.property  name="direcao"
	 */
	public void setDirecao(float direcao, int index) {
		if(direcao<0)
			this.direcao[index] = direcao + (float)(Math.toDegrees(2*Math.PI));
		else	
			this.direcao[index] = direcao;
	}


	/**
	 * Setter of the property <tt>profundidade</tt>
	 * @param profundidade  The profundidade to set.
	 * @uml.property  name="profundidade"
	 */
	public void setProfundidade(int profundidade) {
		this.profundidade = profundidade;
	}
	
	public boolean hasMovement()
	{
		return ( (isMoving) || 
				 ((!isMoving) && (currentPoint == -1)) );
	}
}