package br.edu.fei.dcc.raciocinioespacial.simulador.perfilprofundidade;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Arc2D;
import java.util.ArrayList;
import java.util.List;

import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Agent2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Pedestrian2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.util.Utils;

public class CampoVisaoInstantaneo {
	private Agent2D observador;
	private List<ObstaculoCampoVisao> objetos;
	private int iTime = 0;
	
	public CampoVisaoInstantaneo(Agent2D observador, List<Pedestrian2D> objetos, int time) {
		super();
		this.observador = observador;
		this.objetos = this.getObjetosDentroCampoVisao(objetos);
		this.iTime = time;
		this.criar();
	}
	private List<ObstaculoCampoVisao> getObjetosDentroCampoVisao(List<Pedestrian2D> objetos) {
		Arc2D obs = this.observador.getCampoVisao2D_Old();
		List<ObstaculoCampoVisao> retorno = new ArrayList<ObstaculoCampoVisao>();
		
		for(Pedestrian2D o : objetos)
		{
			Rectangle rectO = o.getRect();
			if(obs.intersects(rectO))
			{
				Point ptObs = new Point((int)obs.getCenterX(), (int)obs.getCenterY());
				Point ptRect = new Point((int)rectO.getCenterX(), (int)rectO.getCenterY());
				if (Utils.convertPixelsToMeters(ptObs.distance(ptRect) - rectO.width) > 0)
				{
					ObstaculoCampoVisao obst = new ObstaculoCampoVisao(o, rectO);
					retorno.add(obst);
				}
			}
		}
		return retorno;
	}
	private void criar() {
		float initialAngle = (float)this.observador.getCampoVisao2D_Old().getAngleStart();
		float finalAngle   = initialAngle + (float)this.observador.getCampoVisao2D_Old().getAngleExtent();
		
		Arc2D obs = this.observador.getCampoVisao2D_Old();
		
		Point ptObs = new Point((int)obs.getCenterX(),(int)obs.getCenterY());
		
		// For each object inside the observer's field of vision, we set an obstacle type
		for (ObstaculoCampoVisao obst : objetos) {
			Rectangle rectPed = obst.getRect();
			Point ptA = new Point((int)rectPed.getCenterX(),(int)rectPed.getCenterY());
			float fDistance = Utils.convertPixelsToMeters((float)(ptObs.distance(ptA)));
			float fAngleRef = (float)Math.toDegrees(Math.atan((ptA.getY()-ptObs.getY())/(ptA.getX()-ptObs.getX())));
			
			if (ptA.getY() < ptObs.getY())
			{
				if (ptA.getX() < ptObs.getX())
					fAngleRef = 180 - fAngleRef;
				else
					fAngleRef *= -1;
			}
			else
			{
				if (ptA.getX() < ptObs.getX())
					fAngleRef = fAngleRef * (-1) + 180;
				else
					fAngleRef = 360 - fAngleRef;
			}
			
			if (fAngleRef < initialAngle)
				fAngleRef = 360 + fAngleRef;
			
			fAngleRef = fAngleRef - initialAngle;
			
			float fAngleOpen = (float)(Math.toDegrees(2*Math.asin((float)Utils.convertPixelsToMeters(rectPed.getWidth()/2)/fDistance)));
			
			obst.setAngles(fAngleOpen, fAngleRef);
			fDistance -= Utils.convertPixelsToMeters(rectPed.width/2);
			obst.setProfundidade(fDistance);
		}
		/*for(float teta=anguloIncial; teta<=anguloFinal; teta++)
		{
			parteCampo.add(new ParteCampoVisaoInstantaneo(this.instante, teta, observador, objetos));
		}*/
	}
	public int getTimeInstant() {
		return iTime;
	}
	
	public void sortByReferenceAngle()
	{
		// A simply bubble sort !
		int iSize = objetos.size();
		for (int i = 0; i < iSize-1; i++)
		{
			for (int j = 0; j < iSize-i-1; j++) {
				if (objetos.get(j).getReferenceAngle() > objetos.get(j+1).getReferenceAngle())
				{
					ObstaculoCampoVisao oTemp = objetos.get(j);
					objetos.set(j, objetos.get(j+1));
					objetos.set(j+1, oTemp);
			    }
			}
		}
	}
	
	public List<ObstaculoCampoVisao> getObstaculos() {
		return objetos;
	}
}
