package br.edu.fei.dcc.raciocinioespacial.simulador.perfilprofundidade;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.List;

import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Agent2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Pedestrian2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.util.Utils;

public class PerfilProfundidade {
	private static final String INICIO = "leftBorder";
	private static final String FIM = "rightBorder";
	private static final String CABECALHO = "description";
	private static final String BACKGROUND = "background";
	private static final String FOREGROUND = "foreground";
	public static final String EXTENSAO = "pl";
	
	private Agent2D observador;
	private List<Pedestrian2D> objetos;
	private CampoVisaoInstantaneo campoVisao;
	private String sCurrentProfile = "";
	private StringBuilder sbAllProfile = new StringBuilder();
	private Graphics graphProfile;
	private float fCurrentAngle = 0;
	private float fatorX = 0;
	private float fatorY = 0;
	private Rectangle rectDepthProfile;
	private Rectangle rectLimiteProfile;
	
	private int iCountCampo = 0;
	
	public PerfilProfundidade(Agent2D observador, List<Pedestrian2D> objetos,
			Graphics g, Rectangle rectDepthProfile, Rectangle rectLimiteProfile) {
		super();
		this.observador = observador;
		this.rectDepthProfile = rectDepthProfile;
		this.rectLimiteProfile = rectLimiteProfile;
		this.graphProfile = g;
		this.objetos = objetos;
		
		float fDepthObserver = Utils.convertPixelsToMeters(observador.getProfundidade()/2);
		
		this.fatorX = (float)(rectDepthProfile.width/(float)(observador.getAngulo()));
		this.fatorY = (float)(rectDepthProfile.height/fDepthObserver);
	}
	public List<Pedestrian2D> getObjetos() {
		return objetos;
	}
	public Agent2D getObservador() {
		return observador;
	}
	
	public CampoVisaoInstantaneo getCampoVisaoInstantaneo()
	{
		if (campoVisao == null)
			calcCampoVisaoInstantaneo();
		
		return campoVisao;
	}
	
	private void makeDepthProfile()
	{
		sCurrentProfile = "";
		fCurrentAngle = 0;
		plotarEsqueletoPerfilProfundidade();
		// Para cada obtculo devidamente calculado no campoVisao
		// vou criando o perfil de profundidade contnuo
		
		// Ordeno por angulo de referencia
		campoVisao.sortByReferenceAngle();
		
		// Verifico oclusoes antes q os msm sejam criados no perfil final de forma erronea
		ObstaculoCampoVisao prevObstatulo = null;
		for (ObstaculoCampoVisao obstaculo : campoVisao.getObstaculos())
		{
			// No caso de ter ocluso, verifico quem est mais perto do agente e
			// retiro a diferenca entre o angulo inicial de um com o final do outro
			// de forma que o angulo de abertura passa a desconsiderar a parte q o agente
			// nao estaria msm vendo de fato
			if (prevObstatulo != null)
				prevObstatulo.trataOclusion(obstaculo);
			
			obstaculo.trataPartialVision(observador.getAngulo());
			
			prevObstatulo = obstaculo;
		}
		
		// Vou criando o perfil...
		prevObstatulo = null;
		for (ObstaculoCampoVisao obstaculo : campoVisao.getObstaculos())
		{
			// Se algum obstculo acabar ficando com 0 graus de angulo de abertura,
			// o msm deve ser ignorado pois est totalmente escondido atrs de um ou mais
			// obstculos
			if (obstaculo.getOpeningAngle() > 0)
			{
				buildPartProfile(obstaculo, prevObstatulo);
				prevObstatulo = obstaculo;
			}
		}
		
		endStringProfile(prevObstatulo);
	}
	
	private void buildPartProfile(ObstaculoCampoVisao obst, ObstaculoCampoVisao prevObst)
	{
		float initialAngle = obst.getInitialAngle();
		float openingAngle = obst.getOpeningAngle();
		float depth = obst.getProfundidade();
		String sObstName = "";
		
		// Plotar rea de silencio
		if (initialAngle > fCurrentAngle)
		{
			float openingAngleAux = initialAngle - fCurrentAngle;
			plotarSilence(openingAngleAux);
			
			if (prevObst != null)
				sObstName = "("+prevObst.getObstaculo().getNome()+","+obst.getObstaculo().getNome()+")";
			else
				sObstName = "("+INICIO+","+obst.getObstaculo().getNome()+")";
			
			addStringProfile(sObstName, -1, openingAngleAux);
		}
		
		sObstName = obst.getObstaculo().getNome();
		plotarPeak(sObstName, openingAngle, depth);
		addStringProfile(sObstName, depth, openingAngle);
		
		fCurrentAngle = initialAngle + openingAngle;
	}
	
	private void addStringProfile(String nameObj, float depth, float angle)
	{
		if (sCurrentProfile == null || sCurrentProfile.length() == 0)
			sCurrentProfile = String.format("%s(t%d,[", CABECALHO, campoVisao.getTimeInstant());
		else
			sCurrentProfile += ",";
		
		if (depth == -1)
			sCurrentProfile += BACKGROUND + "(";
		else
			sCurrentProfile += FOREGROUND + "(";
		
		sCurrentProfile += nameObj.toLowerCase()+","+depth+","+angle+")";
	}
	
	private void endStringProfile(ObstaculoCampoVisao lastObj)
	{
		if (fCurrentAngle < observador.getAngulo())
		{
			float angle = observador.getAngulo() - fCurrentAngle;
			
			String sNameObj = "";
			
			if (lastObj != null)
				sNameObj = "("+lastObj.getObstaculo().getNome()+","+FIM+")";
			else
				sNameObj = "("+INICIO+","+FIM+")";
			
			plotarSilence(angle);
			
			addStringProfile(sNameObj, -1, angle);
		}
		
		sCurrentProfile += "])";
		sbAllProfile = sbAllProfile.append(sCurrentProfile + "\r\n");
	}
	
	private void plotarSilence(float openingAngle)
	{
		if (this.graphProfile != null)
		{
			int iX1 = rectDepthProfile.width + rectDepthProfile.x;
			int iY1 = rectDepthProfile.height + rectDepthProfile.y;
			int iX2 = iX1;
			int iY2 = iY1;

			iX1 -= (int)(fCurrentAngle * fatorX);
			iX2 -= (int)((fCurrentAngle + openingAngle) * fatorX);
			graphProfile.drawLine(iX1, iY1, iX2, iY2);

			fCurrentAngle += openingAngle;
		}
	}
	
	private void plotarPeak(String nameObst, float openingAngle, float depth)
	{
		if (this.graphProfile != null)
		{
			int iX1 = rectDepthProfile.width + rectDepthProfile.x;
			int iY1 = rectDepthProfile.height + rectDepthProfile.y;
			int iX2 = iX1;
			int iY2 = iY1;

			iY2 = rectDepthProfile.y + (int)(depth * fatorY);
			iX1 -= (int)(fCurrentAngle * fatorX);
			iX2 = iX1;
			graphProfile.drawLine(iX1, iY1, iX2, iY2);
			
			graphProfile.drawString(nameObst, iX2-10, iY2-10);

			iY1 = iY2;
			iX2 -= (int)((openingAngle) * fatorX);
			graphProfile.drawLine(iX1, iY1, iX2, iY2);

			iY1 = rectDepthProfile.height + rectDepthProfile.y;
			iX1 = iX2;
			graphProfile.drawLine(iX1, iY1, iX2, iY2);

			fCurrentAngle += openingAngle;
		}
	}
	
	private void plotarEsqueletoPerfilProfundidade()
	{
		if (this.graphProfile != null)
		{
			Graphics2D g2 = (Graphics2D)this.graphProfile;

			g2.setFont(new Font(Font.SERIF, Font.PLAIN, 12));
			g2.clearRect(0, 0, Short.MAX_VALUE, Short.MAX_VALUE);
			g2.setBackground(Color.WHITE);
			g2.draw(rectLimiteProfile);
			g2.setColor(Color.BLACK);
			Utils.drawArrowLine(rectDepthProfile.x-5, rectDepthProfile.y, rectDepthProfile.x-5, rectDepthProfile.y+rectDepthProfile.height+5, 3, 3, g2);
			Utils.drawArrowLine(rectDepthProfile.x-10, rectDepthProfile.y+5, rectDepthProfile.x+rectDepthProfile.width-10, rectDepthProfile.y+5, 3, 3, g2);
		}
	}
	
	public void calcCampoVisaoInstantaneo()
	{
		campoVisao = new CampoVisaoInstantaneo(observador, objetos, iCountCampo);
		makeDepthProfile();
		iCountCampo++;
	}
	public String getCurrentProfile() {
		return sCurrentProfile;
	}
	public StringBuilder getAllProfile() {
		return sbAllProfile;
	}
	public void resetCountCampo()
	{
		iCountCampo = 0;
	}
	public int getCountCampo()
	{
		return iCountCampo;
	}
}
