/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package br.edu.fei.dcc.raciocinioespacial.simulador.prologCommunication;

import java.io.*;
import java.net.*;
import java.util.* ;
import javax.swing.* ; 


/**
  *  A Connector is a server that listens for I/O connections
  *  at a port.  Each client gets a Socket and buffered 
  *  i/o streams bundled in a Transducer object. 
  *  Text read at any input is rebroadcast to other clients.
  *  <pre>
  *  =====================================
  *   Copyright 1999-- John R. Fisher 
  *   jrfisher@csupomona.edu
  *  =====================================  
  *  </pre>
  *  @author jrfisher@csupomona.edu
  */

public class Connector extends Thread { 
   int clientNum ; 
   int port ; 
   ServerSocket portalSocket ; 
   Vector collaborators ; // Object output streams for clients 
   boolean isRunning;
   Socket soc;
   
   public Connector(int port) { 
      this.clientNum = 1 ; 
      this.port = port ; 
      this.collaborators = new Vector() ;
      this.isRunning=true;
      
   }

   public void run() { 
      // Catch big exceptions that prevent server from continuing.
      try { // 1
         portalSocket = new ServerSocket(port) ; 
         while(isRunning) { 
            // Catch smaller exceptions so server itself can continue..
            try { // 2
                //Waits for connection
               this.soc = portalSocket.accept() ;
               this.soc.setSoLinger(true, 0);
               if(isRunning)
               {
               BufferedWriter out = new BufferedWriter(new OutputStreamWriter(soc.getOutputStream())) ;
               collaborators.add(out) ; 
               //System.out.println("Spawning Transducer for " + clientNum) ; 
               Transducer b = 
                  new Transducer(this, 
                               new BufferedReader(new InputStreamReader(soc.getInputStream())), 
                               out, 
                               clientNum, soc) ; 
               b.start() ;
               clientNum++ ;
               }
               else
               {   
                   soc.close();
                   portalSocket.close();
               }
            }
            catch(Exception e2) { 
               JOptionPane.showMessageDialog(null,e2.toString(),"CONNECTOR EXCEPTION #2",JOptionPane.WARNING_MESSAGE) ;
            }
         }
      }
      catch (Exception e1) {
         // Could not make ServerSocket
         JOptionPane.showMessageDialog(null,e1.toString(),"CONNECTOR EXCEPTION #1",JOptionPane.WARNING_MESSAGE) ; 
      }
   } 
}
/**
  *  A Transducer is an attachment to a particular client.
  *  The Transducer listens to this client and rebroadcasts its 
  *  contribution.
  */
class Transducer extends Thread {
   BufferedReader in ; 
   BufferedWriter out ; 
   int client ; 
   Connector portal ; 
   Socket soc;

   Transducer(Connector p, BufferedReader instream,
                    BufferedWriter outstream, int k, Socket socket) { 
      this.portal = p ; 
      in = instream ; 
      out = outstream ; 
      client = k ;
      soc = socket;
   }

   public void run() {
      while(true) { 
         try { 
            // if can read a good input ... 
            String s = in.readLine() ;
            //System.out.println(s+" in Transducer "+client);
            if (s == null || s.equals("end_transducer")) 
            {    // THIS CLIENT IS GONE ...
               synchronized(portal.collaborators)
               {
               this.portal.collaborators.remove(out) ;   // remove this client from server
               //this.out.close();
               //this.in.close();
               //this.soc.close();
               //System.out.println("Transducer "+client+" gone");
               }
               break ;  // stop running this client
            }
            // try to tell everyone ...
            synchronized(portal.collaborators)
            {
                Iterator it = portal.collaborators.iterator() ; 
                while(it.hasNext()) { 
                   // Avoid bad collaborators if possible or necessary ...
                   BufferedWriter bw = null ; 
                   try { 
                      bw = (BufferedWriter)(it.next()) ; 
                      if (bw != out) 
                      {  
                          // don't tell self
                         bw.write(s+"\r") ; // return for readLine at other end
                         bw.flush() ; 
                      }
                   }
                   catch (Exception e2) { 
                      JOptionPane.showMessageDialog(null,e2.toString(),"TRANSDUCER EXCEPTION #2",JOptionPane.WARNING_MESSAGE) ; 
                   }
                }
            }
            if(s.contains("end_prolog"))
            {
                if(client==1)
                {Thread.sleep(1000);} 
                synchronized(portal.collaborators)
                {
               this.portal.collaborators.remove(out) ;   // remove this client from server
               //this.out.close();
               //this.in.close();
               //this.soc.close();
                }
                //System.out.println("Transducer "+client+" gone");
                break ;  // stop running this client
            }
         }   
         // running exception not otherwise handled ...
         catch(Exception e1) {   
            JOptionPane.showMessageDialog(null,e1.toString(),"TRANSDUCER EXCEPTION #1"+client,JOptionPane.WARNING_MESSAGE) ; 
         }
      }  
   }
}





