package br.edu.fei.dcc.raciocinioespacial.simulador.util;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.geom.Arc2D;
import java.awt.geom.RectangularShape;
import java.text.SimpleDateFormat;
import java.util.Calendar;

public class Utils {
	
	private static final float CONST_SCALE = 0.2f;

	public static void aguardar(long milisegundos){
		try {
			Thread.sleep(milisegundos);
		} catch (InterruptedException e) {
			return;
		}
	}

	private static void configGraphics(Graphics g, Stroke s, Color c)
	{
		if(g==null) return;
		Graphics2D g2 = (Graphics2D)g;
		g2.setColor(c);
		g2.setStroke(s);
		
		RenderingHints rh = new RenderingHints(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
		g2.setRenderingHints(rh);
	}
	
	public static void drawArc(Arc2D obj, Graphics g, Color c, Stroke s)
	{
		if(g==null)return;
		Utils.configGraphics(g, s, c);
		((Graphics2D)g).draw(obj);
	}
	
	public static void drawShape(RectangularShape obj,
			boolean isShowBorder, boolean isPaint, Graphics g, Color c, Stroke s)
	{
		if(g==null)return;
		
		if(isShowBorder)
		{
			Utils.configGraphics(g, s, Color.BLACK);
			((Graphics2D)g).draw(obj);
		}
		
		if(isPaint)
		{
			Utils.configGraphics(g, s, c);
			((Graphics2D)g).fill(obj);
		}
	}
	
	public static void drawShape(RectangularShape obj,
			boolean isShowBorder, Graphics g, Color c, Stroke s)
	{
		drawShape(obj, isShowBorder, true, g, c, s);
	}

	public static float getAnguloInclinacaoReta(int x0, int y0, int x1, int y1)
	{
		if((x0-x1)!=0)
		{
			double m = (double)(y0-y1)/(x0-x1);
			return (float)Math.atan(m);
		}
		return 0.0f;
	}
	
	public static String now() {
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat(Messages.getString("Utils.Formato_Completo_Hora")); //$NON-NLS-1$
		return sdf.format(cal.getTime());
	}
	
	public static Polygon removePolygonPointAt(Polygon p, int start, int end)
	{
		Polygon retorno = new Polygon();
		if(p!=null)
		{
			if((start<=p.npoints)&&(start>=0)&&(end<=p.npoints)&&(end>=start))
			{
				for(int i=0; i<p.npoints; i++)
				{
					if((i<start)||(i>end))
					{
						retorno.addPoint(p.xpoints[i], p.ypoints[i]);
					}
				}
			}
		}
		return retorno;
	}
		
	public static void drawArrowLine(int x1, int y1, int x2, int y2, int w, int h, Graphics g)
	{
		g.drawLine(x1, y1, x2, y2);
		if(x1==x2)
		{
			g.drawLine(x2-w, y2-h, x2, y2);
			g.drawLine(x2+w, y2-h, x2, y2);
		}else if(y1==y2)
		{
			g.drawLine(x2-w, y2-h, x2, y2);
			g.drawLine(x2-w, y2+h, x2, y2);
		}
	}
	
	/** Converts a pixel measure in meter measure (constant scale for the sim 5px=1m)*/
	public static float convertPixelsToMeters(float fPixels)
	{
		return fPixels * CONST_SCALE;
	}
	
	public static float convertPixelsToMeters(double dPixels)
	{
		return convertPixelsToMeters((float)dPixels);
	}
	
	/** Converts a meter measure in pixel measure (constant scale for the sim 5px=1m)*/
	public static float convertMetersToPixels(float fMeters)
	{
		return fMeters / CONST_SCALE;
	}
}